library serializer.test.models.book;

import 'package:jaguar_serializer/serializer.dart';
import 'author.dart';

export 'author.dart' show Author, AuthorSerializer;

part 'book.g.dart';

@GenSerializer(addTypeInfo: true)
@ProvideSerializer(Author, AuthorSerializer)
@MapKeyNumToStringProcessor(#publishedDates)
class BookSerializer extends Serializer<Book> with _$BookSerializer {
  Book createModel() => new Book();

  @override
  String typeString() => '$Book';
}

@GenSerializer(addTypeInfo: true)
@ProvideSerializer(Author, AuthorYamlSerializer)
@MapKeyNumToStringProcessor(#publishedDates)
class BookYamlSerializer extends Serializer<Book> with _$BookYamlSerializer {
  Book createModel() => new Book();

  @override
  String typeKey() => '(t)';

  @override
  String typeString() => '$Book';
}

@DefineFieldProcessor()
class MapKeyNumToStringProcessor
    implements FieldProcessor<Map<num, String>, Map<dynamic, String>> {
  final Symbol field;

  const MapKeyNumToStringProcessor(this.field);

  Map<num, String> from(final Map<dynamic, String> input) {
    if (input == null) {
      return null;
    }
    Map<num, String> fromMap = {};
    for (var key in input.keys) {
      if (key is String) {
        fromMap[num.parse(key)] = input[key];
      } else if (key is num) {
        fromMap[key] = input[key];
      }
    }
    return fromMap;
  }

  Map<String, String> to(final Map<num, String> value) {
    if (value == null) {
      return null;
    }
    Map<String, String> toMap = {};
    for (num key in value.keys) {
      toMap[key.toString()] = value[key];
    }
    return toMap;
  }
}

/// Player model for the game
class Book {
  /// Name of the player
  String name;

  List<String> tags;

  Map<num, String> publishedDates;

  List<Author> authors;
}
